@@ -11,13 +11,129 @@ macro_rules! impl_parser {
         where $first: Parser $(, $ty_var : Parser<Input=<$first as Parser>::Input>)* {
         type Input = <$first as Parser>::Input;
         type Output = <$inner_type as Parser>::Output;
-        fn parse_state(&mut self, input: State<<Self as Parser>::Input>) -> ParseResult<<Self as Parser>::Output, <Self as Parser>::Input> {
+        fn parse_state(&mut self, input: State<<Self as Parser>::Input>) -> ParseResult<<Self as Parser>::Output, <Self as Parser>::Input, <Self::Input as Stream>::Item> {
             self.0.parse_state(input)
         }
     }
 }
 }
 
+#[derive(Clone)]
+pub struct Any<I>(PhantomData<fn (I) -> I>);
+
+impl <I> Parser for Any<I>
+    where I: Stream {
+    type Input = I;
+    type Output = I::Item;
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<I::Item, I, I::Item> {
+        input.uncons()
+    }
+}
+
+///Parses any token
+///
+/// ```
+/// # extern crate parser_combinators as pc;
+/// # use pc::*;
+/// # fn main() {
+/// let mut char_parser = any();
+/// assert_eq!(char_parser.parse("!").map(|x| x.0), Ok('!'));
+/// assert!(char_parser.parse("").is_err());
+/// let mut byte_parser = any();
+/// assert_eq!(byte_parser.parse(&b"!"[..]).map(|x| x.0), Ok(&b'!'));
+/// assert!(byte_parser.parse(&b""[..]).is_err());
+/// # }
+/// ```
+pub fn any<I>() -> Any<I>
+    where I: Stream {
+    Any(PhantomData)
+}
+
+
+
+#[derive(Clone)]
+pub struct Satisfy<I, P> { predicate: P, _marker: PhantomData<I> }
+
+impl <I, P> Parser for Satisfy<I, P>
+    where I: Stream, P: FnMut(I::Item) -> bool {
+
+    type Input = I;
+    type Output = I::Item;
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<I::Item, I, I::Item> {
+        match input.clone().uncons() {
+            Ok((c, s)) => {
+                if (self.predicate)(c.clone()) { Ok((c, s)) }
+                else {
+                    Err(Consumed::Empty(ParseError::new(input.position, Error::Unexpected(c))))
+                }
+            }
+            Err(err) => Err(err)
+        }
+    }
+}
+
+///Parses a token and succeeds depending on the result of `predicate`
+///
+/// ```
+/// # extern crate parser_combinators as pc;
+/// # use pc::*;
+/// # fn main() {
+/// let mut parser = satisfy(|c| c == '!' || c == '?');
+/// assert_eq!(parser.parse("!").map(|x| x.0), Ok('!'));
+/// assert_eq!(parser.parse("?").map(|x| x.0), Ok('?'));
+/// # }
+/// ```
+pub fn satisfy<I, P>(predicate: P) -> Satisfy<I, P>
+    where I: Stream, P: FnMut(I::Item) -> bool {
+    Satisfy { predicate: predicate, _marker: PhantomData }
+}
+
+#[derive(Clone)]
+pub struct Token<I>
+    where I: Stream
+        , I::Item: PartialEq {
+    c: I::Item,
+    _marker: PhantomData<I>
+}
+
+impl <I> Parser for Token<I>
+    where I: Stream
+        , I::Item: PartialEq {
+
+    type Input = I;
+    type Output = I::Item;
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<I::Item, I, I::Item> {
+        match input.clone().uncons() {
+            Ok((c, s)) => {
+                if self.c == c { Ok((c, s)) }
+                else {
+                    let errors = vec![Error::Unexpected(c), Error::Expected(self.c.clone().into())];
+                    Err(Consumed::Empty(ParseError::from_errors(input.position, errors)))
+                }
+            }
+            Err(err) => Err(err)
+        }
+    }
+}
+
+///Parses a character and succeeds if the characther is equal to `c`
+///
+/// ```
+/// # extern crate parser_combinators as pc;
+/// # use pc::*;
+/// # fn main() {
+/// let result = token('!')
+///     .parse("!")
+///     .map(|x| x.0);
+/// assert_eq!(result, Ok('!'));
+/// # }
+/// ```
+pub fn token<I>(c: I::Item) -> Token<I>
+    where I: Stream
+        , I::Item: PartialEq {
+    Token { c: c, _marker: PhantomData }
+}
+
 pub struct Choice<S, P>(S, PhantomData<P>);
 
 impl <I, O, S, P> Parser for Choice<S, P>
@@ -26,7 +142,7 @@ impl <I, O, S, P> Parser for Choice<S, P>
         , P: Parser<Input=I, Output=O> {
     type Input = I;
     type Output = O;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I, I::Item> {
         let mut empty_err = None;
         for p in AsMut::as_mut(&mut self.0) {
             match p.parse_state(input.clone()) {
@@ -68,12 +184,13 @@ pub fn choice<S, P>(ps: S) -> Choice<S, P>
 }
 
 #[derive(Clone)]
-pub struct Unexpected<I>(Info, PhantomData<fn (I) -> I>);
+pub struct Unexpected<I>(Info<I::Item>, PhantomData<fn (I) -> I>)
+    where I: Stream;
 impl <I> Parser for Unexpected<I>
     where I : Stream {
     type Input = I;
     type Output = ();
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<(), I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<(), I, I::Item> {
         Err(Consumed::Empty(ParseError::new(input.position, Error::Message(self.0.clone()))))
     }
 }
@@ -93,7 +210,7 @@ impl <I> Parser for Unexpected<I>
 /// ```
 pub fn unexpected<I, S>(message: S) -> Unexpected<I>
     where I: Stream
-        , S: Into<Info> {
+        , S: Into<Info<I::Item>> {
     Unexpected(message.into(), PhantomData)
 }
 
@@ -104,7 +221,7 @@ impl <I, T> Parser for Value<I, T>
         , T: Clone {
     type Input = I;
     type Output = T;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<T, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<T, I, I::Item> {
         Ok((self.0.clone(), Consumed::Empty(input)))
     }
 }
@@ -152,20 +269,22 @@ pub fn not_followed_by<P>(parser: P) -> NotFollowedBy<P>
                  .or(value(())))
 }
 
-struct Iter<P: Parser> {
+pub struct Iter<P: Parser> {
     parser: P,
     input: Consumed<State<P::Input>>,
-    error: Option<Consumed<ParseError>>
+    error: Option<Consumed<ParseError<<P::Input as Stream>::Item>>>
 }
 
 impl <P: Parser> Iter<P> {
     fn new(parser: P, input: State<P::Input>) -> Iter<P> {
         Iter { parser: parser, input: Consumed::Empty(input), error: None }
     }
-    fn into_result<O>(self, result: O) -> ParseResult<O, P::Input> {
+    ///Converts the iterator to a `ParseResult`, returning `Ok` if the parsing so far has be done
+    ///without any errors which consumed data.
+    pub fn into_result<O>(self, value: O) -> ParseResult<O, P::Input, <P::Input as Stream>::Item> {
         match self.error {
             Some(err@Consumed::Consumed(_)) => Err(err),
-            _ => Ok((result, self.input))
+            _ => Ok((value, self.input))
         }
     }
 }
@@ -197,8 +316,8 @@ impl <F, P> Parser for Many<F, P>
     where P: Parser, F: FromIterator<<P as Parser>::Output> {
     type Input = <P as Parser>::Input;
     type Output = F;
-    fn parse_state(&mut self, input: State<<P as Parser>::Input>) -> ParseResult<F, <P as Parser>::Input> {
-        let mut iter = Iter::new(&mut self.0, input);
+    fn parse_state(&mut self, input: State<<P as Parser>::Input>) -> ParseResult<F, <P as Parser>::Input, <Self::Input as Stream>::Item> {
+        let mut iter = (&mut self.0).iter(input);
         let result = iter.by_ref().collect();
         iter.into_result(result)
     }
@@ -232,7 +351,7 @@ impl <F, P> Parser for Many1<F, P>
         , P: Parser {
     type Input = <P as Parser>::Input;
     type Output = F;
-    fn parse_state(&mut self, input: State<<P as Parser>::Input>) -> ParseResult<F, <P as Parser>::Input> {
+    fn parse_state(&mut self, input: State<<P as Parser>::Input>) -> ParseResult<F, <P as Parser>::Input, <Self::Input as Stream>::Item> {
         let (first, input) = try!(self.0.parse_state(input));
 		input.combine(move |input| {
 	        let mut iter = Iter::new(&mut self.0, input);
@@ -318,7 +437,7 @@ impl <F, P, S> Parser for SepBy<F, P, S>
 
     type Input = <P as Parser>::Input;
     type Output = F;
-    fn parse_state(&mut self, input: State<<P as Parser>::Input>) -> ParseResult<F, <P as Parser>::Input> {
+    fn parse_state(&mut self, input: State<<P as Parser>::Input>) -> ParseResult<F, <P as Parser>::Input, <Self::Input as Stream>::Item> {
         let mut input = Consumed::Empty(input);
         let first;
         match input.clone().combine(|input| self.parser.parse_state(input)) {
@@ -352,7 +471,7 @@ impl <F, P, S> Parser for SepBy<F, P, S>
 /// # extern crate parser_combinators as pc;
 /// # use pc::*;
 /// # fn main() {
-/// let result = sep_by(digit(), char(','))
+/// let result = sep_by(digit(), token(','))
 ///     .parse("1,2,3")
 ///     .map(|x| x.0);
 /// assert_eq!(result, Ok(vec!['1', '2', '3']));
@@ -366,10 +485,10 @@ pub fn sep_by<F, P, S>(parser: P, separator: S) -> SepBy<F, P, S>
 }
 
 
-impl <'a, I: Stream, O> Parser for FnMut(State<I>) -> ParseResult<O, I> + 'a {
+impl <'a, I: Stream, O> Parser for FnMut(State<I>) -> ParseResult<O, I, I::Item> + 'a {
     type Input = I;
     type Output = O;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I, I::Item> {
         self(input)
     }
 }
@@ -405,24 +524,24 @@ pub struct FnParser<I, F>(F, PhantomData<fn (I) -> I>);
 /// ```
 pub fn parser<I, O, F>(f: F) -> FnParser<I, F>
     where I: Stream
-        , F: FnMut(State<I>) -> ParseResult<O, I> {
+        , F: FnMut(State<I>) -> ParseResult<O, I, I::Item> {
     FnParser(f, PhantomData)
 }
 
 impl <I, O, F> Parser for FnParser<I, F>
-    where I: Stream, F: FnMut(State<I>) -> ParseResult<O, I> {
+    where I: Stream, F: FnMut(State<I>) -> ParseResult<O, I, I::Item> {
     type Input = I;
     type Output = O;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I, I::Item> {
         (self.0)(input)
     }
 }
 
-impl <I, O> Parser for fn (State<I>) -> ParseResult<O, I>
+impl <I, O> Parser for fn (State<I>) -> ParseResult<O, I, I::Item>
     where I: Stream {
     type Input = I;
     type Output = O;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I, I::Item> {
         self(input)
     }
 }
@@ -433,7 +552,7 @@ impl <P> Parser for Optional<P>
     where P: Parser {
     type Input = <P as Parser>::Input;
     type Output = Option<<P as Parser>::Output>;
-    fn parse_state(&mut self, input: State<<P as Parser>::Input>) -> ParseResult<Option<<P as Parser>::Output>, <P as Parser>::Input> {
+    fn parse_state(&mut self, input: State<<P as Parser>::Input>) -> ParseResult<Option<<P as Parser>::Output>, <P as Parser>::Input, <Self::Input as Stream>::Item> {
         match self.0.parse_state(input.clone()) {
             Ok((x, rest)) => Ok((Some(x), rest)),
             Err(err@Consumed::Consumed(_)) => return Err(err),
@@ -467,7 +586,7 @@ impl_parser! { Between(L, R, P), Skip<With<L, P>, R> }
 /// # extern crate parser_combinators as pc;
 /// # use pc::*;
 /// # fn main() {
-/// let result = between(char('['), char(']'), string("rust"))
+/// let result = between(token('['), token(']'), string("rust"))
 ///     .parse("[rust]")
 ///     .map(|x| x.0);
 /// assert_eq!(result, Ok("rust"));
@@ -491,7 +610,7 @@ impl <I, O, P, Op> Parser for Chainl1<P, Op>
 
     type Input = I;
     type Output = O;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I, I::Item> {
         let (mut l, mut input) = try!(self.0.parse_state(input));
         loop {
             let was_empty = input.is_empty();
@@ -530,7 +649,7 @@ impl <I, O, P, Op> Parser for Chainr1<P, Op>
 
     type Input = I;
     type Output = O;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I, I::Item> {
         let (mut l, mut input) = try!(self.0.parse_state(input));
         loop {
             let was_empty = input.is_empty();
@@ -577,7 +696,7 @@ impl <I, O, P> Parser for Try<P>
 
     type Input = I;
     type Output = O;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I, I::Item> {
         self.0.parse_state(input)
             .map_err(Consumed::as_empty)
     }
@@ -608,7 +727,7 @@ impl <I, A, B, P1, P2> Parser for And<P1, P2>
 
     type Input = I;
     type Output = (A, B);
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<(A, B), I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<(A, B), I, I::Item> {
         let (a, rest) = try!(self.0.parse_state(input));
         rest.combine(move |rest| {
             let (b, rest) = try!(self.1.parse_state(rest));
@@ -624,7 +743,7 @@ impl <I, P1, P2> Parser for With<P1, P2>
 
     type Input = I;
     type Output = <P2 as Parser>::Output;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<<Self as Parser>::Output, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<<Self as Parser>::Output, I, I::Item> {
         let ((_, b), rest) = try!((&mut self.0).and(&mut self.1).parse_state(input));
         Ok((b, rest))
     }
@@ -637,20 +756,21 @@ impl <I, P1, P2> Parser for Skip<P1, P2>
 
     type Input = I;
     type Output = <P1 as Parser>::Output;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<<Self as Parser>::Output, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<<Self as Parser>::Output, I, I::Item> {
         let ((a, _), rest) = try!((&mut self.0).and(&mut self.1).parse_state(input));
         Ok((a, rest))
     }
 }
 
 #[derive(Clone)]
-pub struct Message<P>(P, Info) where P: Parser;
+pub struct Message<P>(P, Info<<P::Input as Stream>::Item>)
+    where P: Parser;
 impl <I, P> Parser for Message<P>
     where I: Stream, P: Parser<Input=I> {
 
     type Input = I;
     type Output = <P as Parser>::Output;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<<Self as Parser>::Output, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<<Self as Parser>::Output, I, I::Item> {
         match self.0.parse_state(input.clone()) {
             Ok(x) => Ok(x),
             Err(err@Consumed::Consumed(_)) => Err(err),
@@ -669,7 +789,7 @@ impl <I, O, P1, P2> Parser for Or<P1, P2>
 
     type Input = I;
     type Output = O;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I, I::Item> {
         match self.0.parse_state(input.clone()) {
             Ok(x) => Ok(x),
             Err(err@Consumed::Consumed(_)) => Err(err),
@@ -691,7 +811,7 @@ impl <I, A, B, P, F> Parser for Map<P, F>
 
     type Input = I;
     type Output = B;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<B, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<B, I, I::Item> {
         match self.0.parse_state(input.clone()) {
             Ok((x, input)) => Ok(((self.1)(x), input)),
             Err(err) => Err(err)
@@ -708,7 +828,7 @@ impl <P, N, F> Parser for Then<P, F>
 
     type Input = <N as Parser>::Input;
     type Output = <N as Parser>::Output;
-    fn parse_state(&mut self, input: State<<Self as Parser>::Input>) -> ParseResult<<Self as Parser>::Output, <Self as Parser>::Input> {
+    fn parse_state(&mut self, input: State<<Self as Parser>::Input>) -> ParseResult<<Self as Parser>::Output, <Self as Parser>::Input, <Self::Input as Stream>::Item> {
         let (value, input) = try!(self.0.parse_state(input));
         input.combine(move |input| {
             let mut next = (self.1)(value);
@@ -718,13 +838,14 @@ impl <P, N, F> Parser for Then<P, F>
 }
 
 #[derive(Clone)]
-pub struct Expected<P>(P, Info);
+pub struct Expected<P>(P, Info<<P::Input as Stream>::Item>)
+    where P: Parser;
 impl <P> Parser for Expected<P>
     where P: Parser {
 
     type Input = <P as Parser>::Input;
     type Output = <P as Parser>::Output;
-    fn parse_state(&mut self, input: State<<Self as Parser>::Input>) -> ParseResult<<Self as Parser>::Output, <Self as Parser>::Input> {
+    fn parse_state(&mut self, input: State<<Self as Parser>::Input>) -> ParseResult<<Self as Parser>::Output, <Self as Parser>::Input, <Self::Input as Stream>::Item> {
         match self.0.parse_state(input) {
             Ok(x) => Ok(x),
             Err(err@Consumed::Consumed(_)) => Err(err),
@@ -740,11 +861,11 @@ pub struct AndThen<P, F>(P, F);
 impl <P, F, O, E> Parser for AndThen<P, F>
     where P: Parser
         , F: FnMut(P::Output) -> Result<O, E>
-        , E: Into<Error> {
+        , E: Into<Error<<P::Input as Stream>::Item>> {
 
     type Input = <P as Parser>::Input;
     type Output = O;
-    fn parse_state(&mut self, input: State<<Self as Parser>::Input>) -> ParseResult<O, <Self as Parser>::Input> {
+    fn parse_state(&mut self, input: State<<Self as Parser>::Input>) -> ParseResult<O, <Self as Parser>::Input, <Self::Input as Stream>::Item> {
         self.0.parse_state(input)
             .and_then(|(o, input)|
                 match (self.1)(o) {
@@ -766,7 +887,7 @@ pub trait ParserExt : Parser + Sized {
     /// # use pc::*;
     /// # fn main() {
     /// let result = digit()
-    ///     .with(char('i'))
+    ///     .with(token('i'))
     ///     .parse("9i")
     ///     .map(|x| x.0);
     /// assert_eq!(result, Ok('i'));
@@ -785,7 +906,7 @@ pub trait ParserExt : Parser + Sized {
     /// # use pc::*;
     /// # fn main() {
     /// let result = digit()
-    ///     .skip(char('i'))
+    ///     .skip(token('i'))
     ///     .parse("9i")
     ///     .map(|x| x.0);
     /// assert_eq!(result, Ok('9'));
@@ -805,7 +926,7 @@ pub trait ParserExt : Parser + Sized {
     /// # use pc::*;
     /// # fn main() {
     /// let result = digit()
-    ///     .and(char('i'))
+    ///     .and(token('i'))
     ///     .parse("9i")
     ///     .map(|x| x.0);
     /// assert_eq!(result, Ok(('9', 'i')));
@@ -886,7 +1007,7 @@ pub trait ParserExt : Parser + Sized {
     /// # use pc::*;
     /// # use pc::primitives::Error;
     /// # fn main() {
-    /// let result = char('9')
+    /// let result = token('9')
     ///     .message("Not a nine")
     ///     .parse("8");
     /// assert!(result.is_err());
@@ -895,7 +1016,7 @@ pub trait ParserExt : Parser + Sized {
     /// # }
     /// ```
     fn message<S>(self, msg: S) -> Message<Self>
-        where S: Into<Info> {
+        where S: Into<Info<<Self::Input as Stream>::Item>> {
         Message(self, msg.into())
     }
 
@@ -907,7 +1028,7 @@ pub trait ParserExt : Parser + Sized {
     /// # use pc::*;
     /// # use pc::primitives::Error;
     /// # fn main() {
-    /// let result = char('9')
+    /// let result = token('9')
     ///     .expected("9")
     ///     .parse("8");
     /// assert!(result.is_err());
@@ -916,7 +1037,7 @@ pub trait ParserExt : Parser + Sized {
     /// # }
     /// ```
     fn expected<S>(self, msg: S) -> Expected<Self>
-        where S: Into<Info> {
+        where S: Into<Info<<Self::Input as Stream>::Item>> {
         Expected(self, msg.into())
     }
 
@@ -935,24 +1056,83 @@ pub trait ParserExt : Parser + Sized {
     /// ```
     fn and_then<F, O, E>(self, f: F) -> AndThen<Self, F>
         where F: FnMut(Self::Output) -> Result<O, E>
-            , E: Into<Error> {
+            , E: Into<Error<<Self::Input as Stream>::Item>> {
         AndThen(self, f)
     }
+
+    ///Creates an iterator from a parser and a state. Can be used as an alternative to `many` when
+    ///collecting directly into a `FromIterator` type is not desirable
+    ///
+    /// ```
+    /// # extern crate parser_combinators as pc;
+    /// # use pc::*;
+    /// # fn main() {
+    /// let mut buffer = String::new();
+    /// let number = parser(|input| {
+    ///     buffer.clear();
+    ///     let mut iter = digit().iter(input);
+    ///     buffer.extend(&mut iter);
+    ///     let i = buffer.parse::<i32>().unwrap();
+    ///     iter.into_result(i)
+    /// });
+    /// let result = sep_by(number, char(','))
+    ///     .parse("123,45,6");
+    /// assert_eq!(result, Ok((vec![123, 45, 6], "")));
+    /// # }
+    /// ```
+    fn iter(self, input: State<Self::Input>) -> Iter<Self> {
+        Iter::new(self, input)
+    }
 }
 
 impl <P: Parser> ParserExt for P { }
 
+macro_rules! tuple_parser {
+    ($($id: ident),+) => {
+        impl <Input: Stream, $($id: Parser<Input=Input>),+> Parser for ($($id),+) {
+            type Input = Input;
+            type Output = ($($id::Output),+);
+            #[allow(non_snake_case)]
+            fn parse_state(&mut self, input: State<Input>) -> ParseResult<($($id::Output),+), Input, Input::Item> {
+                let ($(ref mut $id),+) = *self;
+                let input = Consumed::Empty(input);
+                $(let ($id, input) = try!(input.combine(|input| $id.parse_state(input)));)+
+                Ok((($($id),+), input))
+            }
+        }
+    }
+}
+
+tuple_parser!(A, B);
+tuple_parser!(A, B, C);
+tuple_parser!(A, B, C, D);
+tuple_parser!(A, B, C, D, E);
+tuple_parser!(A, B, C, D, E, F);
+tuple_parser!(A, B, C, D, E, F, G);
+tuple_parser!(A, B, C, D, E, F, G, H);
+tuple_parser!(A, B, C, D, E, F, G, H, I);
+tuple_parser!(A, B, C, D, E, F, G, H, I, J);
+tuple_parser!(A, B, C, D, E, F, G, H, I, J, K);
+tuple_parser!(A, B, C, D, E, F, G, H, I, J, K, L);
+
 #[cfg(test)]
 mod tests {
     use super::*;
     use primitives::Parser;
-    use char::{char, digit};
+    use char::{digit, letter};
 
     #[test]
     fn chainr1_test() {
-        let number = digit().map(|c| c.to_digit(10).unwrap() as i32);
-        let pow = char('^').map(|_| |l:i32, r:i32| l.pow(r as u32));
+        fn pow(l: i32, r: i32) -> i32 { l.pow(r as u32) }
+
+        let number = digit::<&str>().map(|c| c.to_digit(10).unwrap() as i32);
+        let pow = token('^').map(|_| pow);
         let mut parser = chainr1(number, pow);
         assert_eq!(parser.parse("2^3^2"), Ok((512, "")));
     }
+    #[test]
+    fn tuple() {
+        let mut parser = (digit(), token(','), digit(), token(','), letter());
+        assert_eq!(parser.parse("1,2,z"), Ok((('1', ',', '2', ',', 'z'), "")));
+    }
 }
