@@ -37,7 +37,7 @@
 //!     let mut integer_list = sep_by(integer, spaces.skip(char(',')));
 //! 
 //!     //Call parse with the input to execute the parser
-//!     let result: Result<(Vec<i32>, &str), ParseError> = integer_list.parse(input);
+//!     let result: Result<(Vec<i32>, &str), ParseError<char>> = integer_list.parse(input);
 //!     match result {
 //!         Ok((value, _remaining_input)) => println!("{:?}", value),
 //!         Err(err) => println!("{}", err)
@@ -64,36 +64,44 @@
 //! enum Expr {
 //!     Id(String),
 //!     Array(Vec<Expr>),
+//!     Pair(Box<Expr>, Box<Expr>)
 //! }
+//!
 //! fn expr<I>(input: State<I>) -> ParseResult<Expr, I>
 //!     where I: Stream<Item=char> {
 //!     let word = many1(letter());
-//!     let comma_list = sep_by(parser(expr), char(','));
-//!     let array = between(char('['), char(']'), comma_list);
-//!     let spaces = spaces();
-//!     spaces.clone().with(
-//!             word.map(Expr::Id)
-//!             .or(array.map(Expr::Array))
-//!         ).parse_state(input)
+//!     //Creates a parser which parses a char and skips any trailing whitespace
+//!     let lex_char = |c| char(c).skip(spaces());
+//!     let comma_list = sep_by(parser(expr::<I>), lex_char(','));
+//!     let array = between(lex_char('['), lex_char(']'), comma_list);
+//!     //We can use tuples to run several parsers in sequence
+//!     //The resulting type is a tuple containing each parsers output
+//!     let pair = (lex_char('('), parser(expr::<I>), lex_char(','), parser(expr::<I>), lex_char(')'))
+//!         .map(|t| Expr::Pair(Box::new(t.1), Box::new(t.3)));
+//!     word.map(Expr::Id)
+//!         .or(array.map(Expr::Array))
+//!         .or(pair)
+//!         .skip(spaces())
+//!         .parse_state(input)
 //! }
 //! 
 //! fn main() {
 //!     let result = parser(expr)
-//!         .parse("[[], hello, [world]]");
+//!         .parse("[[], (hello, world), [rust]]");
 //!     let expr = Expr::Array(vec![
 //!           Expr::Array(Vec::new())
-//!         , Expr::Id("hello".to_string())
-//!         , Expr::Array(vec![Expr::Id("world".to_string())])
+//!         , Expr::Pair(Box::new(Expr::Id("hello".to_string())),
+//!                      Box::new(Expr::Id("world".to_string())))
+//!         , Expr::Array(vec![Expr::Id("rust".to_string())])
 //!     ]);
 //!     assert_eq!(result, Ok((expr, "")));
 //! }
 //!```
 
 #[doc(inline)]
-pub use primitives::{Parser, ParseResult, ParseError, from_iter};
+pub use primitives::{Parser, ParseError, from_iter};
 #[doc(inline)]
 pub use char::{
-    any_char,
     char,
     digit,
     space,
@@ -108,10 +116,12 @@ pub use char::{
     hex_digit,
     oct_digit,
     string,
-    satisfy,
+
+    ParseResult//use char::ParseResult for compatibility
 };
 #[doc(inline)]
 pub use combinator::{
+    any,
     between,
     chainl1,
     choice,
@@ -119,9 +129,11 @@ pub use combinator::{
     many1,
     optional,
     parser,
+    satisfy,
     sep_by,
     skip_many,
     skip_many1,
+    token,
     try,
     value,
     unexpected,
@@ -184,11 +196,8 @@ mod tests {
     fn field() {
         let word = || many(alpha_num());
         let spaces = spaces();
-        let c_decl = word()
-            .skip(spaces.clone())
-            .skip(char(':'))
-            .skip(spaces)
-            .and(word())
+        let c_decl = (word(), spaces.clone(), char(':'), spaces, word())
+            .map(|t| (t.0, t.4))
             .parse("x: int");
         assert_eq!(c_decl, Ok((("x".to_string(), "int".to_string()), "")));
     }
@@ -198,9 +207,8 @@ mod tests {
 r"
 123
 ";
-        let result = spaces()
-            .with(parser(integer))
-            .skip(spaces())
+        let result = (spaces(), parser(integer), spaces())
+            .map(|t| t.1)
             .parse_state(State::new(source));
         let state = Consumed::Consumed(State {
             position: SourcePosition { line: 3, column: 1 },
@@ -281,18 +289,19 @@ r"
         let m = format!("{}", result.unwrap_err());
 let expected =
 r"Parse error at line: 2, column: 1
-Unexpected character ','
+Unexpected token ','
 Expected 'identifier', 'integer', '[' or '('
 ";
         assert_eq!(m, expected);
     }
 
     fn term(input: State<&str>) -> ParseResult<Expr, &str> {
-
+        fn times(l: Expr, r: Expr) -> Expr { Expr::Times(Box::new(l), Box::new(r)) }
+        fn plus(l: Expr, r: Expr) -> Expr { Expr::Plus(Box::new(l), Box::new(r)) }
         let mul = char('*')
-            .map(|_| |l, r| Expr::Times(Box::new(l), Box::new(r)));
+            .map(|_| times);
         let add = char('+')
-            .map(|_| |l, r| Expr::Plus(Box::new(l), Box::new(r)));
+            .map(|_| plus);
         let factor = chainl1(parser(expr), mul);
         chainl1(factor, add)
             .parse_state(input)
@@ -315,7 +324,7 @@ r"
 
 
     fn follow(input: State<&str>) -> ParseResult<(), &str> {
-        match input.clone().uncons_char() {
+        match input.clone().uncons() {
             Ok((c, _)) => {
                 if c.is_alphanumeric() {
                     Err(Consumed::Empty(ParseError::new(input.position, Error::Unexpected(c))))
@@ -356,7 +365,8 @@ r"
     }
     #[test]
     fn chainl1_error_consume() {
-        let mut p = chainl1(string("abc"), char(',').map(|_| |l, _| l));
+        fn first<T, U>(t: T, _: U) -> T { t }
+        let mut p = chainl1(string("abc"), char(',').map(|_| first));
         assert!(p.parse("abc,ab").is_err());
     }
 
