@@ -1,7 +1,8 @@
 use std::fmt;
 use std::error::Error as StdError;
+use std::any::Any;
 
-///Struct which represents the positions in the source file
+///Struct which represents a position in a source file
 #[derive(Clone, Copy, Debug, Eq, PartialEq, Ord, PartialOrd)]
 pub struct SourcePosition {
     ///Current line of the input
@@ -9,31 +10,35 @@ pub struct SourcePosition {
     ///Current column of the input
     pub column: i32
 }
-impl SourcePosition {
-    fn start() -> SourcePosition {
-        SourcePosition { line: 1, column: 1 }
-    }
 
-    fn update(&mut self, c: &char) {
-        self.column += 1;
-        if *c == '\n' {
-            self.column = 1;
-            self.line += 1;
-        }
+///Struct which represents a position in a byte stream
+#[derive(Clone, Copy, Debug, Eq, PartialEq, Ord, PartialOrd)]
+pub struct BytePosition {
+    ///Current position
+    pub position: usize
+}
+
+impl fmt::Display for BytePosition {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        write!(f, "position: {}", self.position)
     }
 }
 
+///Enum holding error information
+///As there is implementations of `From` for `T: Positioner`, `String` and `&'static str` the
+///constructor need not be used directly as calling `msg.into()` should turn a message into the
+///correct `Info` variant
 #[derive(Clone, Debug)]
-pub enum Info {
-    Token(char),
+pub enum Info<T> {
+    Token(T),
     Owned(String),
     Borrowed(&'static str)
 }
 
-impl PartialEq for Info {
-    fn eq(&self, other: &Info) -> bool {
+impl <T: PartialEq> PartialEq for Info<T> {
+    fn eq(&self, other: &Info<T>) -> bool {
         match (self, other) {
-            (&Info::Token(l), &Info::Token(r)) => l == r,
+            (&Info::Token(ref l), &Info::Token(ref r)) => l == r,
             (&Info::Owned(ref l), &Info::Owned(ref r)) => l == r,
             (&Info::Borrowed(ref l), &Info::Owned(ref r)) => l == r,
             (&Info::Owned(ref l), &Info::Borrowed(ref r)) => l == r,
@@ -42,51 +47,51 @@ impl PartialEq for Info {
         }
     }
 }
-impl fmt::Display for Info {
+impl <T: fmt::Display> fmt::Display for Info<T> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
-            Info::Token(c) => write!(f, "{}", c),
+            Info::Token(ref c) => write!(f, "{}", c),
             Info::Owned(ref s) => write!(f, "{}", s),
             Info::Borrowed(s) => write!(f, "{}", s),
         }
     }
 }
 
-impl From<char> for Info {
-    fn from(s: char) -> Info {
+impl <T: Positioner> From<T> for Info<T> {
+    fn from(s: T) -> Info<T> {
         Info::Token(s)
     }
 }
 
-impl From<String> for Info {
-    fn from(s: String) -> Info {
+impl <T> From<String> for Info<T> {
+    fn from(s: String) -> Info<T> {
         Info::Owned(s)
     }
 }
 
-impl From<&'static str> for Info {
-    fn from(s: &'static str) -> Info {
+impl <T> From<&'static str> for Info<T> {
+    fn from(s: &'static str) -> Info<T> {
         Info::Borrowed(s)
     }
 }
 
 ///Enum used to store information about an error that has occured
 #[derive(Debug)]
-pub enum Error {
+pub enum Error<T> {
     ///Error indicating an unexpected token has been encountered in the stream
-    Unexpected(char),
+    Unexpected(T),
     ///Error indicating that the parser expected something else
-    Expected(Info),
+    Expected(Info<T>),
     ///Generic message
-    Message(Info),
+    Message(Info<T>),
     ///Variant for containing other types of errors
     Other(Box<StdError+Send>)
 }
 
-impl PartialEq for Error {
-    fn eq(&self, other: &Error) -> bool {
+impl <T: PartialEq> PartialEq for Error<T> {
+    fn eq(&self, other: &Error<T>) -> bool {
         match (self, other) {
-            (&Error::Unexpected(l), &Error::Unexpected(r)) => l == r,
+            (&Error::Unexpected(ref l), &Error::Unexpected(ref r)) => l == r,
             (&Error::Expected(ref l), &Error::Expected(ref r)) => l == r,
             (&Error::Message(ref l), &Error::Message(ref r)) => l == r,
             _ => false
@@ -94,8 +99,8 @@ impl PartialEq for Error {
     }
 }
 
-impl <E> From<E> for Error where E: StdError + 'static + Send {
-    fn from(e: E) -> Error {
+impl <E, T> From<E> for Error<T> where E: StdError + 'static + Send {
+    fn from(e: E) -> Error<T> {
         Error::Other(Box::new(e))
     }
 }
@@ -179,8 +184,9 @@ impl <T> Consumed<T> {
     /// assert_eq!(result, Ok((r#"abc"\"#.to_string(), "")));
     /// }
     ///```
-    pub fn combine<F, U, I>(self, f: F) -> ParseResult<U, I>
-        where F: FnOnce(T) -> ParseResult<U, I> {
+    pub fn combine<F, U, I>(self, f: F) -> ParseResult<U, I, I::Item>
+        where F: FnOnce(T) -> ParseResult<U, I, I::Item>
+            , I: Stream {
         match self {
             Consumed::Consumed(x) => {
                 match f(x) {
@@ -196,36 +202,36 @@ impl <T> Consumed<T> {
 ///Struct which hold information about an error that occured at a specific position.
 ///Can hold multiple instances of `Error` if more that one error occured at the position.
 #[derive(Debug, PartialEq)]
-pub struct ParseError {
+pub struct ParseError<P: Positioner> {
     ///The position where the error occured
-    pub position: SourcePosition,
+    pub position: P::Position,
     ///A vector containing specific information on what errors occured at `position`
-    pub errors: Vec<Error>
+    pub errors: Vec<Error<P>>
 }
 
-impl ParseError {
-    pub fn new(position: SourcePosition, error: Error) -> ParseError {
+impl <P: Positioner> ParseError<P> {
+    pub fn new(position: P::Position, error: Error<P>) -> ParseError<P> {
         ParseError::from_errors(position, vec![error])
     }
-    pub fn from_errors(position: SourcePosition, errors: Vec<Error>) -> ParseError {
+    pub fn from_errors(position: P::Position, errors: Vec<Error<P>>) -> ParseError<P> {
         ParseError { position: position, errors: errors }
     }
     pub fn add_message<S>(&mut self, message: S)
-        where S: Into<Info> {
+        where S: Into<Info<P>> {
         self.add_error(Error::Message(message.into()));
     }
-    pub fn add_error(&mut self, message: Error) {
+    pub fn add_error(&mut self, message: Error<P>) {
         //Don't add duplicate errors
         if self.errors.iter().find(|msg| **msg == message).is_none() {
             self.errors.push(message);
         }
     }
-    pub fn set_expected(&mut self, message: Info) {
+    pub fn set_expected(&mut self, message: Info<P>) {
         //Remove all other expected messages
         self.errors.retain(|e| match *e { Error::Expected(_) => false, _ => true });
         self.errors.push(Error::Expected(message));
     }
-    pub fn merge(mut self, other: ParseError) -> ParseError {
+    pub fn merge(mut self, other: ParseError<P>) -> ParseError<P> {
         use std::cmp::Ordering;
         //Only keep the errors which occured after consuming the most amount of data
         match self.position.cmp(&other.position) {
@@ -241,11 +247,14 @@ impl ParseError {
     }
 }
 
-impl StdError for ParseError {
+impl <P> StdError for ParseError<P>
+    where P: fmt::Display + fmt::Debug + Positioner + Any
+        , P::Position: fmt::Display + fmt::Debug + Positioner + Any {
     fn description(&self) -> &str { "parse error" }
 }
 
-impl fmt::Display for ParseError {
+impl <P: Positioner + fmt::Display> fmt::Display for ParseError<P>
+    where P::Position: fmt::Display {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         try!(writeln!(f, "Parse error at {}", self.position));
 
@@ -299,10 +308,10 @@ impl fmt::Display for SourcePosition {
         write!(f, "line: {}, column: {}", self.line, self.column)
     }
 }
-impl fmt::Display for Error {
+impl <T: fmt::Display> fmt::Display for Error<T> {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match *self {
-            Error::Unexpected(c) => write!(f, "Unexpected character '{}'", c),
+            Error::Unexpected(ref c) => write!(f, "Unexpected token '{}'", c),
             Error::Expected(ref s) => write!(f, "Expected {}", s),
             Error::Message(ref msg) => write!(f, "{}", msg),
             Error::Other(ref err) => err.fmt(f)
@@ -312,54 +321,48 @@ impl fmt::Display for Error {
 
 ///The `State<I>` struct keeps track of the current position in the stream `I`
 #[derive(Clone, PartialEq, Debug)]
-pub struct State<I> {
-    pub position: SourcePosition,
+pub struct State<I>
+    where I: Stream {
+    pub position: <<I as Stream>::Item as Positioner>::Position,
     pub input: I
 }
 
 impl <I: Stream> State<I> {
     pub fn new(input: I) -> State<I> {
-        State { position: SourcePosition::start(), input: input }
+        State { position: <I::Item as Positioner>::start(), input: input }
     }
 
     pub fn as_empty(&self) -> State<I> {
-        State { position: self.position, input: self.input.clone() }
+        State { position: self.position.clone(), input: self.input.clone() }
     }
 
     ///`uncons` is the most general way of extracting and item from a stream
     ///It takes a function `f` as argument which should update the position
     ///according to the item that was extracted
     ///Usually you want to use `uncons_char` instead which works directly on character streams
-    pub fn uncons<F>(self, f: F) -> ParseResult<<I as Stream>::Item, I>
-        where F: FnOnce(&mut SourcePosition, &<I as Stream>::Item) {
+    pub fn uncons(self) -> ParseResult<I::Item, I, I::Item> {
         let State { mut position, input, .. } = self;
         match input.uncons() {
             Ok((c, input)) => {
-                f(&mut position, &c);
+                c.update(&mut position);
                 Ok((c, Consumed::Consumed(State { position: position, input: input })))
             }
             Err(()) => Err(Consumed::Empty(ParseError::new(position, Error::Message("End of input".into()))))
         }
     }
 }
-impl <I: Stream<Item=char>> State<I> {
-    ///Specialized uncons function for character streams which updates the position
-    ///with no further action needed
-    pub fn uncons_char(self) -> ParseResult<<I as Stream>::Item, I> {
-        self.uncons(SourcePosition::update)
-    }
-
-}
 
 ///A type alias over the specific `Result` type used by parsers to indicate wether they were
 ///successful or not.
 ///`O` is the type that is output on success
 ///`I` is the specific stream type used in the parser
-pub type ParseResult<O, I> = Result<(O, Consumed<State<I>>), Consumed<ParseError>>;
+///`T` is the item type of `I`, this parameter will be removed once type declarations are allowed
+///to have trait bounds
+pub type ParseResult<O, I, T> = Result<(O, Consumed<State<I>>), Consumed<ParseError<T>>>;
 
 ///A stream is a sequence of items that can be extracted one by one
 pub trait Stream : Clone {
-    type Item;
+    type Item: Positioner;
     ///Takes a stream and removes its first item, yielding the item and the rest of the elements
     ///Returns `Err` when no more elements could be retrieved
     fn uncons(self) -> Result<(Self::Item, Self), ()>;
@@ -375,7 +378,8 @@ impl <'a> Stream for &'a str {
     }
 }
 
-impl <'a, T> Stream for &'a [T] {
+impl <'a, T> Stream for &'a [T]
+    where T: Positioner {
     type Item = &'a T;
     fn uncons(self) -> Result<(&'a T, &'a [T]), ()> {
         if self.len() > 0 {
@@ -387,6 +391,8 @@ impl <'a, T> Stream for &'a [T] {
     }
 }
 
+///Wrapper around iterators which allows them to be treated as a stream.
+///Returned by `from_iter`.
 #[derive(Clone, Debug)]
 pub struct IteratorStream<I>(I)
     where I: Iterator + Clone;
@@ -398,7 +404,8 @@ pub fn from_iter<I>(iter: I) -> IteratorStream<I>
     IteratorStream(iter)
 }
 
-impl <I: Iterator + Clone> Stream for IteratorStream<I> {
+impl <I: Iterator + Clone> Stream for IteratorStream<I>
+    where I::Item: Positioner {
     type Item = <I as Iterator>::Item;
     fn uncons(mut self) -> Result<(I::Item, Self), ()> {
         match self.0.next() {
@@ -408,6 +415,53 @@ impl <I: Iterator + Clone> Stream for IteratorStream<I> {
     }
 }
 
+///`Positioner` represents the operations needed to update a position given an item from the stream
+///When implementing stream for custom token type this must be implemented for that token to allow
+///the position to be updated
+pub trait Positioner: Clone + PartialEq {
+    type Position: Clone + Ord + fmt::Debug;
+    ///Creates a start position
+    fn start() -> Self::Position;
+    ///Updates the position given that `self` has been taken from the stream
+    fn update(&self, position: &mut Self::Position);
+}
+impl <'a, T> Positioner for &'a T
+    where T: Positioner {
+    type Position = <T as Positioner>::Position;
+    fn start() -> <T as Positioner>::Position {
+        <T as Positioner>::start()
+    }
+    fn update(&self, position: &mut <T as Positioner>::Position) {
+        (*self).update(position)
+    }
+}
+
+impl Positioner for char {
+    type Position = SourcePosition;
+    fn start() -> SourcePosition {
+        SourcePosition { line: 1, column: 1 }
+    }
+    fn update(&self, position: &mut SourcePosition) {
+        position.column += 1;
+        if *self == '\n' {
+            position.column = 1;
+            position.line += 1;
+        }
+    }
+}
+
+impl Positioner for u8 {
+    type Position = BytePosition;
+
+    fn start() -> BytePosition {
+        BytePosition { position: 0 }
+    }
+
+    fn update(&self, b: &mut BytePosition) {
+        b.position += 1;
+    }
+}
+
 ///By implementing the `Parser` trait a type says that it can be used to parse an input stream into
 ///the type `Output`.
 pub trait Parser {
@@ -419,7 +473,7 @@ pub trait Parser {
 
     ///Entrypoint of the parser
     ///Takes some input and tries to parse it returning a `ParseResult`
-    fn parse(&mut self, input: Self::Input) -> Result<(Self::Output, Self::Input), ParseError> {
+    fn parse(&mut self, input: Self::Input) -> Result<(Self::Output, Self::Input), ParseError<<Self::Input as Stream>::Item>> {
         match self.parse_state(State::new(input)) {
             Ok((v, state)) => Ok((v, state.into_inner().input)),
             Err(error) => Err(error.into_inner())
@@ -427,13 +481,13 @@ pub trait Parser {
     }
     ///Parses using the state `input` by calling Stream::uncons one or more times
     ///On success returns `Ok((value, new_state))` on failure it returns `Err(error)`
-    fn parse_state(&mut self, input: State<Self::Input>) -> ParseResult<Self::Output, Self::Input>;
+    fn parse_state(&mut self, input: State<Self::Input>) -> ParseResult<Self::Output, Self::Input, <Self::Input as Stream>::Item>;
 }
 impl <'a, I, O, P: ?Sized> Parser for &'a mut P 
     where I: Stream, P: Parser<Input=I, Output=O> {
     type Input = I;
     type Output = O;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I, I::Item> {
         (*self).parse_state(input)
     }
 }
@@ -441,7 +495,7 @@ impl <I, O, P: ?Sized> Parser for Box<P>
     where I: Stream, P: Parser<Input=I, Output=O> {
     type Input = I;
     type Output = O;
-    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I> {
+    fn parse_state(&mut self, input: State<I>) -> ParseResult<O, I, I::Item> {
         (**self).parse_state(input)
     }
 }
